@@ -48,6 +48,7 @@ class Component():
         self._variables: dict[str, ScalarVariable]
 
         self._read_fmu(properties)
+        self._read_step_size(properties)
 
         self._read_generate_proxy(properties)
         self._read_remote_access(properties)
@@ -76,6 +77,11 @@ class Component():
         if self.fmu.default_experiment and not self.step_size:
             self.step_size = self.fmu.default_experiment.step_size
 
+    def _read_step_size(self, properties: MutableMapping):
+        if 'stepSize' not in properties:
+            return
+        self.step_size = float(properties['stepSize'])
+
     def _read_initialize(self, properties: MutableMapping):
         if 'initialize' not in properties:
             return
@@ -96,6 +102,8 @@ class Component():
             connector = Connector(name=connector_name)
             if 'variable' in connector_properties:
                 connector.variable = connector_properties['variable']
+            if 'variableGroup' in connector_properties:
+                connector.variable_group = connector_properties['variableGroup']
             if 'type' in connector_properties:
                 connector.type = connector_properties['type']
             self._connectors[connector.name] = connector
@@ -138,6 +146,7 @@ class Component():
 
     def _init_variables(self):
         self._variables = deepcopy(self.fmu.variables)
+
         for variable_name, variable in self._initial_values.items():
             if variable.causality:
                 self._variables[variable_name].causality = variable.causality
@@ -167,6 +176,7 @@ class Component():
         """
         osp_model_description_file = self.fmu.file.parent.absolute(
         ) / f'{self.name}_OspModelDescription.xml'
+        self._clean(osp_model_description_file)
 
         osp_model_description = {}
 
@@ -246,6 +256,13 @@ class Component():
 
         DictWriter.write(osp_model_description, osp_model_description_file)
 
+    def _clean(self, file_to_remove: Union[str, Path]):
+        """Clean up single file
+        """
+        if isinstance(file_to_remove, str):
+            file_to_remove = Path.cwd() / file_to_remove
+        file_to_remove.unlink(missing_ok=True)
+
 
 @dataclass()
 class RemoteAccess():
