@@ -68,11 +68,10 @@ class OspCaseBuilder():
         if inspect:
             # inspect and stop
             case._inspect()
-            # clean up
-            case.clean()
+
             return
 
-        case.write_osp_model_description_xmls()
+        # case.write_osp_model_description_xmls()
         case.write_osp_system_structure_xml()
         case.write_system_structure_ssd()
 
