@@ -1,7 +1,8 @@
 import logging
 import re
 from pathlib import Path
-from shutil import rmtree, copy2
+from shutil import copy2
+from typing import Union
 
 from dictIO import CppDict, DictWriter, XmlFormatter
 from dictIO.utils.counter import BorgCounter
@@ -58,9 +59,6 @@ class OspSimulationCase():
         """
         logger.info(f"Set up OSP simulation case '{self.name}' in case folder: {self.case_folder}")
 
-        # Clean up case folder
-        self.clean()
-
         # Register and copy to local case folder all FMUs referenced by components in the case dict
         self._copy_fmus_from_library()
 
@@ -75,34 +73,12 @@ class OspSimulationCase():
         # Make sure all components have a step size defined
         self._check_components_step_size()
 
-    def clean(self):
-        """Cleans up the case folder and deletes any existing ospx files, e.g. modelDescription.xml .fmu .csv etc.
+    def _clean(self, file_to_remove: Union[str, Path]):
+        """Clean up single file
         """
-
-        # specify all files to be deleted (or comment-in / comment-out as needed)
-        case_builder_result_files = [
-            '*.csv',
-            '*.out',
-            '*.xml',
-            '*.fmu',
-            '*callGraph',
-            '*.pdf',
-            #'*.png',                   # 'protect results/*.png'
-            'watchDict',
-            'statisticsDict',           # 'results',
-            'zip',
-        ]
-
-        logger.info(f'Clean OSP simulation case folder: {self.case_folder}')
-
-        for pattern in case_builder_result_files:
-            files = list(Path('.').rglob(pattern))
-            for file in files:
-                if file.is_file():
-                    if not file.name.startswith('test_'):
-                        file.unlink(missing_ok=True)
-                else:
-                    rmtree(file)
+        if isinstance(file_to_remove, str):
+            file_to_remove = self.case_folder / file_to_remove
+        file_to_remove.unlink(missing_ok=True)
 
     def write_osp_model_description_xmls(self):
         """Writes the <component.name>_OspModelDescription.xml files for all components defined in the system structure
@@ -120,7 +96,11 @@ class OspSimulationCase():
     def write_osp_system_structure_xml(self):
         """Writes the OspSystemStructure.xml file
         """
-        # sourcery skip: merge-dict-assign
+        # sourcery skip: class-extract-method, merge-dict-assign
+
+        osp_system_structure_file = self.case_folder / 'OspSystemStructure.xml'
+        self._clean(osp_system_structure_file)
+
         logger.info(
             f"Write OspSystemStructure.xml file for OSP simulation case '{self.name}' in case folder: {self.case_folder}"
         )
@@ -146,13 +126,19 @@ class OspSimulationCase():
         simulators: dict = {}
         for index, (_, component) in enumerate(self.system_structure.components.items()):
             simulator_key = f'{index:06d}_Simulator'
-            simulator_properties = {
+            simulator_properties: dict[str, dict[str, Union[str, float, dict]]] = {
                 '_attributes': {
                     'name': component.name,
                     'source': component.fmu.file.name,
-                    'stepSize': component.step_size
                 }
             }
+            if component.step_size:
+                write_step_size_to_osp_system_structure: bool = True
+                if component.fmu.default_experiment and component.fmu.default_experiment.step_size and component.step_size == component.fmu.default_experiment.step_size:
+                    write_step_size_to_osp_system_structure = False
+                if write_step_size_to_osp_system_structure:
+                    simulator_properties['_attributes']['stepSize'] = component.step_size
+
             if component.initial_values:
                 simulator_properties['InitialValues'] = {}
                 for index, (_, variable) in enumerate(component.initial_values.items()):
@@ -183,29 +169,47 @@ class OspSimulationCase():
         # Connections
         connections: dict = {}
         for connection in self.system_structure.connections.values():
-            if connection.source and connection.target:
+            if not connection.is_valid:
+                continue
+            if connection.is_variable_connection:
                 connection_key = f'{self.counter():06d}_VariableConnection'
                 # (note: the order 000000, 000001 is essential here!)
                 connections[connection_key] = {
                     '000000_Variable': {
                         '_attributes': {
-                            'simulator': connection.source.component,
-                            'name': connection.source.variable,
+                            'simulator': connection.source_endpoint.component.name,
+                            'name': connection.source_endpoint.variable_name,
                         }
                     },
                     '000001_Variable': {
                         '_attributes': {
-                            'simulator': connection.target.component,
-                            'name': connection.target.variable,
+                            'simulator': connection.target_endpoint.component.name,
+                            'name': connection.target_endpoint.variable_name,
+                        }
+                    }
+                }
+            if connection.is_variable_group_connection:
+                connection_key = f'{self.counter():06d}_VariableGroupConnection'
+                # (note: the order 000000, 000001 is essential here!)
+                connections[connection_key] = {
+                    '000000_VariableGroup': {
+                        '_attributes': {
+                            'simulator': connection.source_endpoint.component.name,
+                            'name': connection.source_endpoint.variable_name,
+                        }
+                    },
+                    '000001_VariableGroup': {
+                        '_attributes': {
+                            'simulator': connection.target_endpoint.component.name,
+                            'name': connection.target_endpoint.variable_name,
                         }
                     }
                 }
         osp_system_structure['Connections'] = connections
 
         # Write OspSystemStructure.xml
-        target_file = self.case_folder / 'OspSystemStructure.xml'
         formatter = XmlFormatter()
-        DictWriter.write(osp_system_structure, target_file, formatter=formatter)
+        DictWriter.write(osp_system_structure, osp_system_structure_file, formatter=formatter)
 
         self._correct_wrong_xml_namespace(
             'OspSystemStructure.xml',
@@ -218,6 +222,10 @@ class OspSimulationCase():
     def write_system_structure_ssd(self):
         """Writes the SystemStructure.ssd file
         """
+
+        system_structure_ssd_file = self.case_folder / 'SystemStructure.ssd'
+        self._clean(system_structure_ssd_file)
+
         # sourcery skip: merge-dict-assign
         logger.info(
             f"Write SystemStructure.ssd file for OSP simulation case '{self.name}' in case folder: {self.case_folder}"
@@ -288,22 +296,21 @@ class OspSimulationCase():
         # Connections
         connections: dict = {}
         for connection in self.system_structure.connections.values():
-            if connection.source and connection.target:
+            if connection.source_endpoint and connection.target_endpoint:
                 connection_key = f'{self.counter():06d}_Connection'
                 connections[connection_key] = {
                     '_attributes': {
-                        'startElement': connection.source.component,
-                        'startConnector': connection.source.variable,
-                        'endElement': connection.target.component,
-                        'endConnector': connection.target.variable,
+                        'startElement': connection.source_endpoint.component.name,
+                        'startConnector': connection.source_endpoint.variable_name,
+                        'endElement': connection.target_endpoint.component.name,
+                        'endConnector': connection.target_endpoint.variable_name,
                     }
                 }
         system_structure_ssd['System']['Connections'] = connections
 
         # Write SystemStructure.ssd
-        target_file_path = Path.cwd() / 'SystemStructure.ssd'
         formatter = XmlFormatter(omit_prefix=False)
-        DictWriter.write(system_structure_ssd, target_file_path, formatter=formatter)
+        DictWriter.write(system_structure_ssd, system_structure_ssd_file, formatter=formatter)
 
         return
 
@@ -312,6 +319,9 @@ class OspSimulationCase():
 
         I.e. for documentation or further statistical analysis.
         """
+        statistics_dict_file = self.case_folder / 'statisticsDict'
+        # self._clean(statistics_dict_file)
+
         # sourcery skip: merge-dict-assign, simplify-dictionary-update
         logger.info(
             f"Write statistics dict for OSP simulation case '{self.name}' in case folder: {self.case_folder}"
@@ -359,9 +369,7 @@ class OspSimulationCase():
             'names': list(self.system_structure.variables.keys()),
         }
 
-        target_file_path = Path.cwd() / 'statisticsDict'
-
-        DictWriter.write(statistics_dict, target_file_path, mode='a')
+        DictWriter.write(statistics_dict, statistics_dict_file, mode='a')
 
     def write_watch_dict(self):
         """Writes a case-specific watch dict file
@@ -371,6 +379,9 @@ class OspSimulationCase():
             - convergence plotting
             - extracting the results
         """
+        watch_dict_file = self.case_folder / 'watchDict'
+        # self._clean(watch_dict_file)
+
         logger.info(
             f"Write watch dict for OSP simulation case '{self.name}' in case folder: {self.case_folder}"
         )
@@ -388,15 +399,13 @@ class OspSimulationCase():
             no_of_connectors = len(component.connectors.keys())
 
             # @TODO: Time, StepCount, conn0, conn1, etc from modelDescription.xml ModelVariables
-            #        should match connectors in caseDict for respective model.Improvement needed.
+            #        should match connectors in caseDict for respective model. Improvement needed.
             #        FRALUM, 2021-xx-xx
-            # columns = [0, 1]+[x+2 for x in range(no_of_connectors)]
             columns = [0, 1] + [x + 2 for x in range(no_of_connectors)]     # f*** StepCount
 
             watch_dict['datasources'].update({component_name: {'columns': columns}})
 
-        target_file_path = Path.cwd() / 'watchDict'
-        DictWriter.write(watch_dict, target_file_path, mode='a')
+        DictWriter.write(watch_dict, watch_dict_file, mode='a')
 
         return
 
@@ -504,11 +513,12 @@ class OspSimulationCase():
             )
             log_string += fmu_attributes
             if default_experiment_key := find_key(fmu.model_description, 'DefaultExperiment$'):
-                fmu_default_experiment = '\n'.join(
-                    f'\t{delim}{k}{delim}{v}' for k,
-                    v in fmu.model_description[default_experiment_key]['_attributes'].items()
-                )
-                log_string += f'\n{fmu_default_experiment}'
+                if '_attributes' in fmu.model_description[default_experiment_key]:
+                    fmu_default_experiment = '\n'.join(
+                        f'\t{delim}{k}{delim}{v}' for k,
+                        v in fmu.model_description[default_experiment_key]['_attributes'].items()
+                    )
+                    log_string += f'\n{fmu_default_experiment}'
         logger.info(log_string + '\n')
 
         log_string = (
@@ -541,13 +551,13 @@ class OspSimulationCase():
 
         log_string = (
             f"Connectors defined in {self.case_dict.name}\n"
-            f"\tComponent{delim}Connector{delim}Variable{delim}Type"
+            f"\tComponent{delim}Connector{delim}Variable{delim}VariableGroup{delim}Type"
         )
         for component_name, component in self.system_structure.components.items():
             if component.connectors:
                 log_string += f'\n\n\t{component_name}\n'
                 connector_definitions = '\n'.join(
-                    f'\t{delim}{connector_name}{delim}{connector.variable}{delim}{connector.type}'
+                    f'\t{delim}{connector_name}{delim}{connector.variable}{delim}{connector.variable_group}{delim}{connector.type}'
                     for connector_name,
                     connector in component.connectors.items()
                 )
@@ -561,8 +571,12 @@ class OspSimulationCase():
     def _write_plot_config_json(self):
         """Writes the PlotConfig.json file, containing postprocessing information
         """
-        temp_dict = {'plots': []}
+
+        plot_config_file = self.case_folder / 'PlotConfig.json'
+        self._clean(plot_config_file)
+
         if 'plots' in self.case_dict['postProcessing'].keys():
+            temp_dict = {'plots': []}
             for plot in self.case_dict['postproc']['plots'].values():
                 variables: list[dict] = []
                 for component_name, component in self.system_structure.components.items():
@@ -584,8 +598,7 @@ class OspSimulationCase():
                     }
                 )
 
-            target_file_path = Path.cwd() / 'PlotConfig.json'
-            DictWriter.write(temp_dict, target_file_path)
+            DictWriter.write(temp_dict, plot_config_file)
 
         return
 
