@@ -1,7 +1,7 @@
 import logging
 import re
 from pathlib import Path
-from typing import MutableSequence
+from typing import List, MutableSequence
 
 import matplotlib
 import matplotlib.axes
@@ -121,23 +121,28 @@ class CosimWatcher:
                 current_key = list(df)[index + 1
                                        ]            # 0 is Time, StepCount was removed for simplification
 
-                subplot: matplotlib.axes.SubplotBase = self.figure.add_subplot(
-                    self.maxRow, self.number_of_columns, index + 1
-                )
+                subplot: Union[matplotlib.axes.Axes,
+                               matplotlib.axes.SubplotBase] = self.figure.add_subplot(
+                                   self.maxRow, self.number_of_columns, index + 1
+                               )
                 subplot.plot(
                     'Time',
                     current_key,
-                    linewidth = 2,
-                    color = cm.get_cmap('gist_rainbow')(index / self.number_of_subplots),
-                    data = df[['Time', current_key]]
+                    linewidth=2,
+                    color=cm.get_cmap('gist_rainbow')(index / self.number_of_subplots),
+                    data=df[['Time', current_key]]
                 )
                 # subplot.set_title(currentKey,  fontsize=10)
                 subplot.grid(color='#66aa88', linestyle='--')
                 subplot.xaxis.set_tick_params(labelsize=8)
                 subplot.yaxis.set_tick_params(labelsize=8)
                 subplot.legend(fontsize=8)
-
-                axs.append(subplot)
+                if isinstance(subplot, matplotlib.axes.SubplotBase):
+                    axs.append(subplot)
+                else:
+                    raise TypeError(
+                        f'CosimWatcher.plot(): subplot is of type {type(subplot)}. Expected type was matplotlib.axes.SubplotBase .'
+                    )
 
             self.figure.suptitle(self.title)
 
@@ -166,15 +171,15 @@ class CosimWatcher:
         df = self._read_csv_files_into_dataframe()
 
         result_dict = {}
-        for item in list(df):
-            arr = df[item].to_numpy()
-            result_dict[item] = {
-                'latestValue': arr[-1],
-                'firstValue': arr[0],
-                'mean': np.mean(arr),
-                'stdev': np.std(arr),
-                'min': np.min(arr),
-                'max': np.max(arr)
+        for header in list(df):
+            values = df[header].dropna().to_numpy()
+            result_dict[header] = {
+                'latestValue': values[-1],
+                'firstValue': values[0],
+                'mean': np.mean(values),
+                'stdev': np.std(values),
+                'min': np.min(values),
+                'max': np.max(values)
             }
 
         # debug
@@ -197,36 +202,56 @@ class CosimWatcher:
         """
 
         pattern = re.compile(r'(^#|\s+\[.*?\]$)')
-        #pattern = re.compile(r'(^#|\s+\[[\w\d\s]+\]$)')
+        # pattern = re.compile(r'(^#|\s+\[[\w\d\s]+\]$)')
 
         for data_source_name, data_source_properties in self.data_sources.items():                                    # loop over all data sources
             for csv_file_name in self.csv_file_names:
-                if re.match(data_source_name, csv_file_name):                                       # find the correct csv file
+                if re.match(data_source_name, csv_file_name):   # find the correct csv file
                     data_source_properties.update({'csvFile': csv_file_name})
+
+                    # extract the header row from the csv file to determine the variable names
+                    data_header: List[str] = []
                     with open(csv_file_name, 'r') as f:
-                        data_header = f.readline().strip().split(
-                            self.delimiter
-                        )                                                                           # extract the header line to find the variable names
-                        if 'columns' in data_source_properties:                                     # if key columns was given
-                            colNames = [data_header[x] for x in data_source_properties['columns']]
-                            data_source_properties.update({'colNames':colNames})
+                        data_header = f.readline().strip().split(self.delimiter)
+                    if not data_header:
+                        continue
+                    columns: List[int] = []
+                    read_only_shortlisted_columns: bool = False
+                    # @TODO: The following line is commented out, currently,
+                    #        as the column indices shortlisted in watch dict
+                    #        are incorrect if variableGroups are used in connectors.
+                    #        Once this issue is solved and shortlisted columns are correctly written to the watch dict again,
+                    #        the following line can be activated and this TODO comment be deleted.
+                    #        CLAROS, 2022-09-26
+                    # read_only_shortlisted_columns = True if 'columns' in data_source_properties else False
+                    if read_only_shortlisted_columns:
+                        # if columns were explicitely specified (shortlisted) in watch dict:
+                        # Read only shortlisted columns.
+                        columns = data_source_properties['columns']
+                    else:
+                        # if columns were not explicitely specified in watch dict:
+                        # Read all columns except settings.
+                        for index, col_name in enumerate(data_header):
+                            # if not re.match(r'^(StepCount|settings)', col_name):
+                            if not re.match(r'^(settings)', col_name):
+                                columns.append(index)
 
-                            displayColNames = [pattern.sub('', col_name) for col_name in data_source_properties['colNames']]
-                            displayColNames = ['Time', 'StepCount'] + [data_source_name + '|' + col_name for col_name in displayColNames if col_name not in ['Time', 'StepCount']]
-                            data_source_properties.update({'displayColNames':displayColNames})
+                    _column_names = [data_header[column] for column in columns]
+                    data_source_properties.update({'colNames': _column_names})
 
-                        else:                                                                       # if no columns is given, extract all relevant columns avoiding settings. and StepCount
-                            colNames = [col_name for col_name in data_header if not re.match(r'^(StepCount|settings)', col_name)]
-                            data_source_properties.update({'colNames':colNames})
+                    _display_column_names = [
+                        pattern.sub('', col_name)
+                        for col_name in data_source_properties['colNames']
+                    ]
+                    _display_column_names = ['Time', 'StepCount'] + [
+                        data_source_name + '|' + col_name
+                        for col_name in _display_column_names
+                        if col_name not in ['Time', 'StepCount']
+                    ]
+                    data_source_properties.update({'displayColNames': _display_column_names})
 
-                            displayColNames = [pattern.sub('', col_name) for col_name in data_source_properties['colNames']]
-                            displayColNames = ['Time', 'StepCount'] + [data_source_name + '|' + col_name for col_name in displayColNames if col_name not in ['Time', 'StepCount']]
-                            data_source_properties.update({'displayColNames': displayColNames})
-
-                            data_source_properties.update({'columns': list(range(len(data_source_properties['colNames'])))})
-
-                    data_source_properties.update({'xColumn': data_source_properties['columns'][0]})
-                    data_source_properties.update({'yColumns': data_source_properties['columns'][1:]})
+                    data_source_properties.update({'xColumn': columns[0]})
+                    data_source_properties.update({'yColumns': columns[1:]})
 
     def _initialize_plot(self):
         """Initializes the plot.
@@ -267,30 +292,31 @@ class CosimWatcher:
 
         df_all_data_sources = pd.DataFrame()    # initialize empty df
 
-        for index, (data_source_name, data_source_properties) in enumerate(self.data_sources.items()):
-            # create the mapping dict
-            map = dict(zip(data_source_properties['colNames'], data_source_properties['displayColNames']))
+        for data_source_name, data_source_properties in self.data_sources.items():
+            # mapping dict for display column names
+            column_name_to_display_column_name_mapping = dict(
+                zip(data_source_properties['colNames'], data_source_properties['displayColNames'])
+            )
             '''it could be so easy
             but we have to remove Time and StepCount because they are in each csv file and need to be filtered
             could be also required here to specify an abscissa differing from column 1 or 2
             but, anyways this is only applicable to cosim
             '''
-            #for remove_item in ['Time', 'StepCount']:
-            #    map.pop(remove_item, None)
+            # for remove_item in ['Time', 'StepCount']:
+            #    column_name_to_display_column_name_mapping.pop(remove_item, None)
 
-            if index == 0:                                                              # first call, also include Time (and StepCount)
-                df_single_data_source = pd.read_csv(
-                    Path(data_source_properties['csvFile']),
-                    usecols=data_source_properties['colNames'],
-                )
-            else:
-                df_single_data_source = pd.read_csv(
-                    Path(data_source_properties['csvFile']),
-                    #usecols = [col_name for col_name in data_source_properties['colNames'] if col_name not in ['Time', 'StepCount']],
-                    usecols = data_source_properties['colNames'],
-                )
+            _column_names: List[str] = data_source_properties['colNames']
+            # Alternative value for _column_names which excludes 'Time' and 'StepCount':
+            # _column_names = [col_name for col_name in data_source_properties['colNames'] if col_name not in ['Time', 'StepCount']],
 
-            df_single_data_source = df_single_data_source.rename(columns=map)   # rename
+            df_single_data_source = pd.read_csv(
+                Path(data_source_properties['csvFile']),
+                usecols=_column_names,
+            )
+
+            df_single_data_source = df_single_data_source.rename(
+                columns=column_name_to_display_column_name_mapping
+            )
 
             if df_all_data_sources.empty:
                 # first df inherit all columns from single df
@@ -300,23 +326,24 @@ class CosimWatcher:
                 # ignoring index
                 # (after setting individual time steps for each individual component)
 
-                #df_all_data_sources = pd.concat([df_all_data_sources, df_single_data_source], axis=1)   # concatenate column-wise
+                # concatenate column-wise
+                # df_all_data_sources = pd.concat([df_all_data_sources, df_single_data_source], axis=1)
 
-                #df_all_data_sources = df_all_data_sources.append(df_single_data_source, ignore_index=True)
-                df_all_data_sources = df_all_data_sources.append(df_single_data_source)
+                # df_all_data_sources = pd.concat([df_all_data_sources, df_single_data_source], ignore_index=True)
+                df_all_data_sources = pd.concat([df_all_data_sources, df_single_data_source])
 
                 # potential solution
                 # interpolating non-matching time data
                 # otherwise should component-wise dataframes do a better job
                 # bypass StepCound yielding in big holes, not plotted by mpl.
-                #df_all_data_sources = pd.merge_asof(
+                # df_all_data_sources = pd.merge_asof(
                 #    df_all_data_sources,
                 #    df_single_data_source,
                 #    on = 'Time',
                 #    by = 'StepCount',
                 #    direction = 'nearest',
                 #    #tolerance = pd.Timedelta('1ms')
-                #)
+                # )
 
         # find latest common start point for skip and latest
         # consider skipping negative values due to wrong inputs
@@ -341,5 +368,5 @@ class CosimWatcher:
         # Opening and closing of window may be deprecated when a better solution is found
         mgr = plt.get_current_fig_manager()
         mgr.full_screen_toggle()
-        self.screenSize = (mgr.canvas.width(), mgr.canvas.height())
-        mgr.window.close()
+        self.screenSize = (mgr.canvas.width(), mgr.canvas.height())     # type: ignore
+        mgr.window.close()                                              # type: ignore
